/*
 * Copyright (c) 2004-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: NodeGUI.java,v 1.17 2007/03/06 23:05:22 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.graph.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.List;

import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.graph.Node;
import edu.indiana.extreme.xbaya.graph.Port;

/**
 * @author Satoshi Shirasuna
 */
public abstract class NodeGUI implements GraphPieceGUI {

    /**
     * The minimum width of the node.
     */
    protected static final int MINIMUM_WIDTH = 100;

    /**
     * The minimum height of the node
     */
    protected static final int MINIMUM_HEIGHT = 37;

    protected static final int TEXT_GAP_X = 5;

    protected static final int TEXT_GAP_Y = 2;

    protected static final Color TEXT_COLOR = Color.black;

    protected static final int PORT_GAP = 13;

    protected static final int PORT_INITIAL_GAP = 10;

    protected static final Color EDGE_COLOR = Color.black;

    private static final Color DEFAULT_HEAD_COLOR = Color.white;

    private static final Color SELECTED_HEAD_COLOR = Color.pink;

    /**
     * The default body color.
     */
    public static final Color DEFAULT_BODY_COLOR = new Color(250, 220, 100);

    protected static final Color DRAGGED_BODY_COLOR = Color.lightGray;

    protected Node node;

    protected Dimension dimension;

    protected int headHeight;

    private boolean selected = false;

    protected boolean dragged = false;

    protected Color headColor;

    protected Color bodyColor;

    private List<Paintable> paintables;

    /**
     * @param node
     */
    public NodeGUI(Node node) {
        this.node = node;
        this.bodyColor = DEFAULT_BODY_COLOR;
        this.headColor = DEFAULT_HEAD_COLOR;
        // The followings are just to make sure that it has some size.
        this.dimension = new Dimension(MINIMUM_WIDTH, MINIMUM_HEIGHT);
        this.paintables = new LinkedList<Paintable>();
    }

    /**
     * Sets the color of the body.
     * 
     * @param color
     *            The color
     */
    public void setBodyColor(Color color) {
        this.bodyColor = color;
    }

    /**
     * @return The color of the body.
     */
    public Color getBodyColor() {
        return this.bodyColor;
    }

    /**
     * Sets the color of the head.
     * 
     * @param color
     *            The color to set
     */
    public void setHeadColor(Color color) {
        this.headColor = color;
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.gui.GraphPieceGUI#mouseClicked(java.awt.event.MouseEvent,
     *      edu.indiana.extreme.xbaya.XBayaEngine)
     */
    public void mouseClicked(MouseEvent event, XBayaEngine engine) {
        // Nothing by default
    }

    /**
     * @param paintable
     */
    public void addPaintable(Paintable paintable) {
        this.paintables.add(paintable);
    }

    /**
     * @param paintable
     */
    public void removePaintable(Paintable paintable) {
        this.paintables.remove(paintable);
    }

    /**
     * @param flag
     */
    protected void setSelectedFlag(boolean flag) {
        this.selected = flag;
        if (this.selected) {
            this.headColor = SELECTED_HEAD_COLOR;
        } else {
            this.headColor = DEFAULT_HEAD_COLOR;
        }
    }

    /**
     * @param flag
     */
    protected void setDraggedFlag(boolean flag) {
        this.dragged = flag;
    }

    /**
     * Returns the position of the node.
     * 
     * @return the position of the node
     */
    protected Point getPosition() {
        return this.node.getPosition();
    }

    /**
     * Gets the bounding Rectangle of this Node.
     * 
     * @return A rectangle indicating this component's bounds
     */
    protected Rectangle getBounds() {
        return new Rectangle(this.node.getPosition(), this.dimension);
    }

    /**
     * Checks if a user's click is to select the node.
     * 
     * @param point
     *            The location.
     * @return true if the user's click is to select the node; false otherwise
     */
    protected boolean isIn(Point point) {
        Rectangle bounds = getBounds();
        return bounds.contains(point);
    }

    /**
     * Checks if a user's click is to select the configuration
     * 
     * @param point
     * @return true if the user's click is to select the node, false otherwise
     */
    @SuppressWarnings("unused")
    protected boolean isInConfig(Point point) {
        return false;
    }

    /**
     * Calculates the width of the node and x-coordinate of the ports. This
     * method has to be called before painting any parts of the graph.
     * 
     * @param g
     */
    protected void calculatePositions(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        this.headHeight = fm.getHeight() + TEXT_GAP_Y * 2;

        int maxNumPort = Math.max(this.node.getOutputPorts().size(), this.node
                .getInputPorts().size());
        this.dimension.height = Math.max(this.headHeight + PORT_INITIAL_GAP
                + PORT_GAP * maxNumPort, MINIMUM_HEIGHT);
        this.dimension.width = Math.max(MINIMUM_WIDTH, fm.stringWidth(this.node
                .getID())
                + TEXT_GAP_X * 2);

        /* Calculates the position of ports */
        setPortPositions();
    }

    /**
     * @param g
     */
    protected void paint(Graphics2D g) {

        Point position = this.node.getPosition();

        // Draws the body.
        if (this.dragged) {
            g.setColor(DRAGGED_BODY_COLOR);
        } else {
            g.setColor(this.bodyColor);
        }
        g.fillRect(position.x, position.y, this.dimension.width,
                this.dimension.height);

        // Draws the head.
        g.setColor(this.headColor);
        g.fillRect(position.x, position.y, this.dimension.width,
                this.headHeight);

        // Text
        g.setColor(TEXT_COLOR);

        // XXX it's debatable if we should show the ID or the name.
        // String name = this.node.getName();
        String name = this.node.getID();
        g.drawString(name, position.x + TEXT_GAP_X, position.y
                + this.headHeight - TEXT_GAP_Y);

        // Edge
        g.setColor(EDGE_COLOR);
        g.drawRect(position.x, position.y, this.dimension.width,
                this.dimension.height);

        // Paint all ports
        for (Port port : this.node.getAllPorts()) {
            port.getGUI().paint(g);
        }

        // Paint extras
        for (Paintable paintable : this.paintables) {
            paintable.paint(g, this.node.getPosition());
        }
    }

    /**
     * Sets up the position of ports
     */
    protected void setPortPositions() {
        // inputs
        List<? extends Port> inputPorts = this.node.getInputPorts();
        for (int i = 0; i < inputPorts.size(); i++) {
            Port port = inputPorts.get(i);
            Point offset = new Point(PortGUI.DATA_PORT_SIZE / 2,
                    this.headHeight + PORT_INITIAL_GAP + PORT_GAP * i);
            port.getGUI().setOffset(offset);
        }

        // outputs
        List<? extends Port> outputPorts = this.node.getOutputPorts();
        for (int i = 0; i < outputPorts.size(); i++) {
            Port port = outputPorts.get(i);
            // Use getBounds() instead of this.dimension because subclass might
            // overwrite getBounds() to have different shape.
            Point offset = new Point(this.getBounds().width
                    - PortGUI.DATA_PORT_SIZE / 2, this.headHeight
                    + PORT_INITIAL_GAP + PORT_GAP * i);
            port.getGUI().setOffset(offset);
        }

        // control-in
        Port controlInPort = this.node.getControlInPort();
        if (controlInPort != null) {
            controlInPort.getGUI().setOffset(new Point(0, 0));
        }

        // control-outs
        for (Port controlOutPort : this.node.getControlOutPorts()) {
            // By default, all ports will be drawn at the same place. Subclass
            // should rearrange them if there are more than one control-out
            // ports.
            controlOutPort.getGUI().setOffset(
                    new Point(getBounds().width, getBounds().height));
        }
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2004-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
